Package: BgeeCall
Type: Package
Title: Automatic RNA-Seq present/absent gene expression calls
        generation
Version: 1.28.4
Date: 2021-05
Authors@R: 
	c(person("Julien", "Wollbrett", 
		role = c("aut", "cre"), 
		email = "julien.wollbrett@unil.ch"),
    person("Alessandro", "Brandulas Cammarata", role = "aut",
        email = "alessandro.brandulascammarata@unil.ch"),
    person("Sara", "Fonseca Costa", role = "aut"),
    person("Julien", "Roux", role = "aut"),
    person("Marc", "Robinson Rechavi", role = "ctb"),
    person("Frederic", "Bastian", role = "aut"))
Description: BgeeCall allows to generate present/absent gene expression calls without using an arbitrary cutoff like TPM<1. 
    Calls are generated based on reference intergenic sequences. These sequences are generated based on expression of all RNA-Seq 
    libraries of each species integrated in Bgee (https://bgee.org).
Depends: R (>= 3.6)
Imports: AnnotationDbi, curl, ggplot2, scales, GenomicFeatures,
        tximport, Biostrings, readr, sjmisc, RCurl, RSQLite, tools,
        stringr, rtracklayer, jsonlite, methods, dplyr, data.table,
        sjmisc, grDevices, graphics, stats, utils, rslurm, rhdf5,
        txdbmaker, IRanges, spatstat.univar
License: GPL-3 + file LICENSE
URL: https://github.com/BgeeDB/BgeeCall
BugReports: https://github.com/BgeeDB/BgeeCall/issues
VignetteBuilder: knitr
biocViews: Software, GeneExpression, RNASeq
Suggests: knitr, testthat, rmarkdown, AnnotationHub, GenomeInfoDb, httr
Encoding: UTF-8
NeedsCompilation: no
SystemRequirements: kallisto
Roxygen: list(roclets = c("namespace", "rd"))
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/BgeeCall
git_branch: RELEASE_3_23
git_last_commit: 615f78c
git_last_commit_date: 2026-07-06
Repository: Bioconductor 3.23
Date/Publication: 2026-07-07
Packaged: 2026-07-07 20:32:00 UTC; biocbuild
Author: Julien Wollbrett [aut, cre],
  Alessandro Brandulas Cammarata [aut],
  Sara Fonseca Costa [aut],
  Julien Roux [aut],
  Marc Robinson Rechavi [ctb],
  Frederic Bastian [aut]
Maintainer: Julien Wollbrett <julien.wollbrett@unil.ch>
