# Generated by roxygen2: do not edit by hand

export(Pvalue_averaging)
export(create_kallisto_index)
export(download_fasta_intergenic)
export(download_kallisto)
export(generate_calls_workflow)
export(generate_initial_intergenic_regions)
export(generate_presence_absence)
export(generate_reference_intergenic_regions)
export(generate_slurm_calls)
export(generate_slurm_indexes)
export(getIntergenicPrefix)
export(getIntergenicRelease)
export(getRunIds)
export(getSimpleArborescence)
export(getWorkingPath)
export(get_summary_stats)
export(list_bgee_ref_intergenic_species)
export(list_community_ref_intergenic_species)
export(list_intergenic_release)
export(merge_transcriptome_and_intergenic)
export(merging_libraries)
export(run_kallisto)
export(run_tximport)
export(setAnnotationFromFile)
export(setAnnotationFromObject)
export(setIntergenicRelease)
export(setOutputDir)
export(setRNASeqLibPath)
export(setRunIds)
export(setSimpleArborescence)
export(setTranscriptomeFromFile)
export(setTranscriptomeFromObject)
export(setWorkingPath)
exportClasses(KallistoMetadata)
import(rslurm)
importFrom(AnnotationDbi,keys)
importFrom(AnnotationDbi,select)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,collapse)
importFrom(Biostrings,countPattern)
importFrom(Biostrings,intersect)
importFrom(Biostrings,readDNAStringSet)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,setdiff)
importFrom(Biostrings,setequal)
importFrom(Biostrings,subseq)
importFrom(Biostrings,union)
importFrom(Biostrings,width)
importFrom(Biostrings,writeXStringSet)
importFrom(GenomicFeatures,exonsBy)
importFrom(GenomicFeatures,transcriptsBy)
importFrom(IRanges,IRanges)
importFrom(IRanges,coverage)
importFrom(IRanges,intersect)
importFrom(IRanges,slice)
importFrom(RCurl,getURL)
importFrom(RSQLite,SQLite)
importFrom(RSQLite,dbConnect)
importFrom(RSQLite,dbDisconnect)
importFrom(RSQLite,dbGetQuery)
importFrom(curl,curl_download)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,rbindlist)
importFrom(data.table,setDT)
importFrom(data.table,setkey)
importFrom(data.table,setnames)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(jsonlite,fromJSON)
importFrom(methods,"slot<-")
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,validObject)
importFrom(readr,parse_date)
importFrom(readr,read_delim)
importFrom(readr,read_tsv)
importFrom(readr,write_tsv)
importFrom(rhdf5,h5read)
importFrom(rslurm,get_job_status)
importFrom(rslurm,get_slurm_out)
importFrom(rslurm,slurm_apply)
importFrom(rtracklayer,export)
importFrom(rtracklayer,import)
importFrom(scales,alpha)
importFrom(sjmisc,str_contains)
importFrom(spatstat.univar,weighted.median)
importFrom(stats,aggregate)
importFrom(stats,approxfun)
importFrom(stats,complete.cases)
importFrom(stats,density)
importFrom(stats,end)
importFrom(stats,integrate)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,weighted.mean)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_match)
importFrom(tools,file_path_sans_ext)
importFrom(txdbmaker,makeTxDbFromGFF)
importFrom(txdbmaker,makeTxDbFromGRanges)
importFrom(tximport,summarizeToGene)
importFrom(tximport,tximport)
importFrom(utils,compareVersion)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,untar)
importFrom(utils,unzip)
importFrom(utils,write.table)
