#' @importFrom stats density p.adjust pnorm qnorm sd median 
#' @importFrom dplyr %>% arrange bind_rows distinct mutate n pull rename summarise ungroup
#' @importFrom data.table data.table fread fwrite rbindlist setDT setkey setnames
#' @importFrom Biostrings DNAString DNAStringSet readDNAStringSet writeXStringSet width subseq reverseComplement
#' @importFrom IRanges IRanges coverage slice
#' @importFrom txdbmaker makeTxDbFromGFF makeTxDbFromGRanges
#' @importFrom GenomicFeatures exonsBy transcriptsBy
#' @importFrom rtracklayer import export
#' @importFrom readr parse_date read_tsv write_tsv
#' @importFrom sjmisc str_contains
#' @importFrom tximport tximport summarizeToGene
#' @importFrom methods new slot "slot<-" is
#' @importFrom ggplot2 ggplot aes geom_density geom_vline scale_color_manual labs theme_minimal scale_fill_manual theme element_blank element_text element_rect
#' @importFrom grDevices dev.off pdf
#' @importFrom graphics legend lines par plot
#' @importFrom utils read.table write.table
#' @importFrom rslurm slurm_apply get_job_status get_slurm_out
#' @importFrom RCurl getURL
#' @importFrom stringr str_extract str_detect str_match
#' @importFrom tools file_path_sans_ext
#' @importFrom curl curl_download
#' @importFrom jsonlite fromJSON
#' @importFrom graphics arrows axis mtext
#' @importFrom methods callNextMethod validObject
#' @importFrom stats aggregate approxfun complete.cases end integrate na.omit quantile weighted.mean
#' @importFrom utils compareVersion download.file head packageVersion read.delim untar unzip
#' @importFrom spatstat.univar weighted.median
#' @importFrom AnnotationDbi select keys
#' 

NULL 